/**
 * Copyright (c) 
 *
 * All rights reserved.
 *
 */
#ifndef APP_USBD_CCID_INTERNAL_H__
#define APP_USBD_CCID_INTERNAL_H__

#include "app_util.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @defgroup app_usbd_ccid_internals USB ccid internals
 * @ingroup app_usbd_ccid
 *
 * @brief @tagAPI52840 Internals of the USB ccid.
 * @{
 */

/** @brief Forward declaration of ccid Class type */
APP_USBD_CLASS_FORWARD(app_usbd_ccid);

/*lint -save -e165*/
/**
 * @brief Forward declaration of @ref app_usbd_ccid_user_event_e.
 *
 */
enum app_usbd_ccid_user_event_e;

/*lint -restore*/

/**
 * @brief User event handler.
 *
 * @param[in] p_inst    Class instance.
 * @param[in] event     User event.
 *
 */
typedef void (*app_usbd_ccid_user_ev_handler_t)(app_usbd_class_inst_t const *    p_inst,
                                                 enum app_usbd_ccid_user_event_e event);


/** @brief ccid part of class instance data */
typedef struct {
    uint8_t con_interface;     //!< Interface number of ccid.
    uint8_t con_epin;          //!< CCID control IN endpoint.
    uint8_t data_epout;         //!< DATA OUT endpoint.
    uint8_t data_epin;          //!< DATA IN endpoint.	

    app_usbd_ccid_user_ev_handler_t user_ev_handler; //!< User event handler.
    uint8_t * p_ep_interval;    //!< Endpoint intervals.

} app_usbd_ccid_inst_t; /////////////////////

/**
 * @brief CCID class specific request handled via control endpoint.
 */
typedef struct {
    uint8_t type;    //!< Request type.
    uint8_t len;     //!< Request length.

} app_usbd_ccid_req_t; /////usb_device_ccid_control_request_struct_t

/**
 * @brief CCID rx transfer buffer.
 */
typedef struct {
    uint8_t * p_buf;        //!< User buffer pointer.
    size_t    read_left;    //!< Bytes left to read into buffer.
} ccid_rx_buffer_t;

/** @brief ccid context */
typedef struct {
    app_usbd_ccid_req_t     request;             //!< CCID class request.
    //app_usbd_ccid_state_t   state;               //!< Internal module state
	
    ccid_rx_buffer_t rx_transfer[2];                 //!< User receive transfers.

    uint8_t   internal_rx_buf[NRF_DRV_USBD_EPSIZE]; //!< Internal receive buffer.
    uint8_t * p_copy_pos;                           //!< Current copy position from internal buffer.

    size_t  bytes_left;                             //!< Bytes left in internal buffer to copy.
    size_t  bytes_read;                             //!< Bytes currently written to user buffer.
    size_t  last_read;                              //!< Bytes read in last transfer.
    size_t  cur_read;                               //!< Bytes currently read to internal buffer.
} app_usbd_ccid_ctx_t; //////////////////////

/**
 * @brief Default interval value for comm endpoint IN
 *
 */
#define APP_USBD_CCID_DEFAULT_INTERVAL   0x10

/**
 * @brief CCID class configuration macro.
 *
 * Used by @ref APP_USBD_CCID_GLOBAL_DEF
 *
 * @param iface_con   Interface number of ccid.
 * @param epin_con    Control IN endpoint.
 * @param epin_data   DATA IN endpoint.
 * @param epout_data  DATA OUT endpoint.
 *
 */
#define APP_USBD_CCID_CONFIG(iface_con, epin_con, epin_data, epout_data)      \
        ((iface_con, epin_con, epin_data, epout_data))	 

/**
 * @brief Specific class constant data for ccid.
 *
 * @ref app_usbd_ccid_inst_t
 */
#define APP_USBD_CCID_INSTANCE_SPECIFIC_DEC app_usbd_ccid_inst_t inst;

/**
 * @brief Configures ccid class instance.
 *
 * @param user_event_handler    User event handler.
 * @param con_ifc               Interface number of ccid .
 * @param con_ein               Control IN endpoint.
 * @param data_ein              DATA IN endpoint.
 * @param data_eout             DATA OUT endpoint.
 * @param ep_list               List of endpoints and intervals
 */
#define APP_USBD_CCID_INST_CONFIG(user_event_handler,                                            \
                                     con_ifc,                                                    \
                                     con_ein,                                                    \
                                     data_ein,                                                   \
                                     data_eout,                                                  \
                                     ep_list)                                                    \
        .inst = {                                                                                \
                .user_ev_handler = user_event_handler,                                           \
                .con_interface  = con_ifc,                                                       \
                .con_epin       = con_ein,                                                       \
                .data_epin       = data_ein,                                                     \
                .data_epout      = data_eout,                                                    \
                .p_ep_interval   = ep_list                                                       \
        }
		
/**
 * @brief Specific class data for ccid.
 *
 * @ref app_usbd_ccid_ctx_t
 * */
#define APP_USBD_CCID_DATA_SPECIFIC_DEC app_usbd_ccid_ctx_t ctx;

/**
 * @brief Public CCID interface.
 */
//extern const app_usbd_ccid_methods_t app_usbd_ccid_methods;

/** @brief Public ccid class interface */
extern const app_usbd_class_methods_t app_usbd_ccid_class_methods;

/** @brief Global definition of ccid instance */
#define APP_USBD_CCID_GLOBAL_DEF_INTERNAL(instance_name,                                          \
                                             user_ev_handler,                                     \
                                             con_ifc,                                             \
                                             con_ein,                                             \
                                             data_ein,                                            \
                                             data_eout)                                           \
    static uint8_t CONCAT_2(instance_name, _ep) = {                                               \
            (APP_USBD_EXTRACT_INTERVAL_FLAG(con_ein) ? APP_USBD_EXTRACT_INTERVAL_VALUE(con_ein)   \
             : APP_USBD_CCID_DEFAULT_INTERVAL)};                                                  \
    APP_USBD_CLASS_INST_GLOBAL_DEF(                                                               \
        instance_name,                                                                            \
        app_usbd_ccid,                                                                            \
        &app_usbd_ccid_class_methods,                                                             \
        APP_USBD_CCID_CONFIG(con_ifc, con_ein, data_ein, data_eout),                     	  \
        (APP_USBD_CCID_INST_CONFIG(user_ev_handler,                                               \
                                              con_ifc,                                            \
                                              con_ein,                                            \
                                              data_ein,                                           \
                                              data_eout,                                          \
                                              &CONCAT_2(instance_name, _ep)))                     \
                )                                                                                      


/** @} */


#ifdef __cplusplus
}
#endif

#endif /* APP_USBD_CCID_INTERNAL_H__ */
